import pandas as pd
from omnivector.abstraction import AbstractDB

class LanceDB(AbstractDB):
    """
    LanceDB is a vector database that uses Lance to store and search vectors.
    """
    def __init__(self):
        super().__init__()

    def create_index(self, ids, vectors, metadata=None):
        import lance
        import pyarrow as pa
        from lance.vector import vec_to_table


        data = pd.DataFrame({"id": ids})

        if metadata is not None:
            meta_df = pd.DataFrame.from_records(metadata)
            data = pd.concat([data, meta_df], axis=1)
        table = vec_to_table(vectors)

        combined = pa.Table.from_pandas(data).append_column("vector", table["vector"])
        ds = lance.write_dataset(combined, self.config["lancedb"]["DB_PATH"], mode="overwrite")


    def vector_search(self, vector, k=3):
        import lance
        self.ds = lance.dataset(self.config["lancedb"]["DB_PATH"])
        return self.ds.to_table(
            nearest={
                "column": "vector",
                "k": k,
                "q": vector,
                "nprobes": 20,
                "refine_factor": 100
            }).to_pandas()



class AbstractEmbedder:
    def __init__(self, **kwargs):
       pass

class OpenAIEmbedder(AbstractEmbedder):

    def __init__(self):
        super().__init__()

    def embed(self, sentences):
        pass


class SentenceTransformerEmbedder(AbstractEmbedder):
    def __init__(self, model_path, device="cpu"):
        super().__init__()
        from sentence_transformers import SentenceTransformer

        self.model = SentenceTransformer(model_path)
        self.model.to(device)

    def embed(self, sentences):
        return self.model.encode(sentences, show_progress_bar=True)




import itertools
from omnivector.abstraction import AbstractDB
import time

def chunks(iterable, batch_size=100):
    it = iter(iterable)
    chunk = tuple(itertools.islice(it, batch_size))
    while chunk:
        yield chunk
        chunk = tuple(itertools.islice(it, batch_size))


class PineconeDB(AbstractDB):
    def __init__(self,):
        super().__init__()
        import pinecone
        pinecone.init(
            api_key=self.config["pinecone"]['PINECONE_API_KEY'],
            environment=self.config["pinecone"]['GPC_ENVIRONMENT']
        )
        self.index_name = self.config["pinecone"]['INDEX_NAME']

    def create_index(self, ids, vectors, metadata=None):
        import pinecone
        # only create index if it doesn't exist
        if self.index_name not in pinecone.list_indexes():
            pinecone.create_index(
                name=self.index_name,
                dimension=len(vectors[0]),
                metric='cosine'
            )
            # wait a moment for the index to be fully initialized
            time.sleep(1)

        # now connect to the index
        self.index = pinecone.GRPCIndex(self.index_name)

        all_data = []

        for idx, meta, emb in zip(ids, metadata, vectors):
            all_data.append({
                 "id": f"vec_{idx}",
                 "values": emb.tolist(),
                 'metadata': meta})

        for ids_vectors_chunk in chunks(all_data, batch_size=100):
            print(ids_vectors_chunk[0])
            self.index.upsert(vectors=list(ids_vectors_chunk))

    def vector_search(self, vector, k=1):
        # now query
        xc = self.index.query(vector.tolist(), top_k=k, include_metadata=True)
        return xc










from omnivector.abstraction import  AbstractDB

class WeaviateDB(AbstractDB):

    def __init__(self):
        super().__init__()

        import weaviate
        import json

        self.client = weaviate.Client(
            url=self.config["weaviate"]["URL"],  # Replace with your endpoint
            auth_client_secret=weaviate.AuthApiKey(api_key=self.config["weaviate"]["API_KEY"]),
            # Replace w/ your Weaviate instance API key

        )

    def create_index(self, ids, vectors, metadata=None):
        class_obj = {
            "class": self.config["weaviate"]["CLASS"],
            "vectorizer": "none",
            # If set to "none" you must always provide vectors yourself. Could be any other "text2vec-*" also.
        }

        self.client.schema.create_class(class_obj)

        self.client.batch.configure(batch_size=100)  # Configure batch
        with self.client.batch as batch:  # Configure a batch process
            for id, t, v in zip(ids,metadata,vectors):  # Batch import all Questions
                properties = {
                    "index": id,
                    **metadata
                }
                batch.add_data_object(
                    data_object=properties,
                    class_name=self.config["weaviate"]["CLASS"],
                    vector=v  # Add custom vector
                )

    def vector_search(self, vector, k=3):
        import json
        response = (
            self.client.query
            .get(self.config["weaviate"]["CLASS"], ["index", "text"])
            .with_near_vector({
                "vector": vector
            })
            .with_limit(k)
            .with_additional(["distance"])
            .do()
        )

        return json.dumps(response, indent=4)



import pandas as pd
import os
import pinecone
import time


class AbstractDB:

    def __init__(self):
        import yaml
        self.config = yaml.safe_load(open(os.environ["OMNIVECTOR_CONFIG"]))

    def create_index(self, ids, vectors, metadata=None):
        pass

    def vector_search(self, vector, k=1):
        pass






